// CS370 - Fall 2016
// Lab08 - PerspectiveCubes

#ifdef OSX
	#include <GLUT/glut.h>
#else
	#include <GL/glew.h>
	#include <GL/glut.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

// Shader file utility functions
#include "shaderutils.h"

// Shader files
GLchar* vertexFile = "basicvert.vs";
GLchar* fragmentFile = "basicfrag.fs";

// Shader objects
GLuint shaderProg;

#define DEG2RAD 0.01745
#define X 0
#define Y 1
#define Z 2
#define CUBE 1

// Cube vertices
GLfloat cube[][3] = {{-1.0f,-1.0f,-1.0f},{1.0f,-1.0f,-1.0f},{1.0f,-1.0f,1.0f},
                     {-1.0f,-1.0f,1.0f},{-1.0f,1.0f,-1.0f},{1.0f,1.0f,-1.0f},
                     {1.0f,1.0f,1.0f},{-1.0f,1.0f,1.0f}};

// Vertex colors
GLfloat colors[][3] = {{0.0f,0.0f,0.0f},{1.0f,0.0f,0.0f},{1.0f,0.0f,1.0f},
                       {0.0f,0.0f,1.0f},{0.0f,1.0f,0.0f},{1.0f,1.0f,0.0f},
                       {1.0f,1.0f,1.0f},{0.0f,1.0f,1.0f}};

// Global spherical coord values
GLfloat azimuth = 45.0f;
GLfloat daz = 2.0f;
GLfloat elevation = 45.0f;
GLfloat del = 2.0f;
GLfloat radius = 6.0f;
GLfloat dr = 0.1f;
GLfloat min_radius = 2.0f;

// Global camera vectors
GLfloat eye[3] = {4.344f,3.152f,2.682f};
GLfloat at[3] = {0.0f,0.0f,0.0f};
GLfloat up[3] = {0.0f,1.0f,0.0f};

// Animation flag
GLint anim = false;

void display();
void render_Scene();
void keyfunc(unsigned char key, int x, int y);
void reshape(int w, int h);
void colorcube();
void quad(GLfloat v1[], GLfloat v2[], GLfloat v3[], GLfloat v4[],
	      GLfloat c1[], GLfloat c2[], GLfloat c3[], GLfloat c4[]);
void create_lists();

int main(int argc, char *argv[])
{
	// Initialize GLUT
	glutInit(&argc, argv);

	// Initialize the window with double buffering and RGB colors
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);

	// Set the window size to 500x500 pixels
	glutInitWindowSize(500, 500);

	// Create window
	glutCreateWindow("Projection Stack");

#ifndef OSX
	// Initialize GLEW - MUST BE DONE AFTER CREATING GLUT WINDOW
	glewInit();
#endif

	// Define callbacks
	glutDisplayFunc(display);
	glutKeyboardFunc(keyfunc);
	glutReshapeFunc(reshape);

	// Compute initial cartesian camera position
	eye[X] = (GLfloat)(radius*sin(azimuth*DEG2RAD)*sin(elevation*DEG2RAD));
	eye[Y] = (GLfloat)(radius*cos(elevation*DEG2RAD));
	eye[Z] = (GLfloat)(radius*cos(azimuth*DEG2RAD)*sin(elevation*DEG2RAD));

	// TODO: Create display list
	create_lists();

	// Set background color to white
	glClearColor(1.0f,1.0f,1.0f,1.0f);

	// Enable depth test
	glEnable(GL_DEPTH_TEST);

	// Load shader programs
	shaderProg = load_shaders(vertexFile,fragmentFile);
	glUseProgram(shaderProg);

	// Begin event loop
	glutMainLoop();

	return 0;
}

// Display callback
void display()
{
	// Reset background
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// TODO: Set perspective projection matrix
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glFrustum(-1.0f, 1.0f, -1.0f, 1.0f, 1.0f, 8.0f);
	// Alternatively
	// glPerspective(90.0f,1.0f,1.0f,8.0f);
	// glOrtho(-4.0f, 4.0f, -4.0f, 4.0f, 1.0f, 8.0f); // orthographic projection

	// Set modelview matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// TODO: Set camera position
	gluLookAt(eye[X], eye[Y], eye[Z], at[X], at[Y], at[Z], up[X], up[Y], up[Z]);

	// Render scene
	render_Scene();

	// Flush buffer
	glFlush();

	// Swap buffers
	glutSwapBuffers();
}

// Scene render function
void render_Scene()
{
	// TODO: Render cube using display list
	glPushMatrix();
		glTranslatef(0.0f,0.0f,0.0f);
		glRotatef(0.0f,0.0f,0.0f,1.0f);
		glScalef(1.5f, 0.5f, 1.5f);
		glCallList(CUBE);
	glPopMatrix();

	// TODO: Render sphere using glut object
	glPushMatrix();
		glTranslatef(0.0f, 1.5f, 0.0f);
		glRotatef(0.0f,0.0f,0.0f,1.0f);
		glScalef(1.0f,1.0f,1.0f);
		glColor3f(1.0f, 0.0f, 0.0f);
		glutSolidSphere(1.0f, 20, 20);
	glPopMatrix();

}

// Keyboard callback
void keyfunc(unsigned char key, int x, int y)
{
	// TODO: Adjust azimuth angle
	if (key == 'a')
	{
		azimuth += daz;
		if (azimuth > 360.0)
		{
			azimuth -= 360.0;
		}
	}
	else if (key == 'd')
	{
		azimuth -= daz;
		if (azimuth < 0.0)
		{
			azimuth += 360.0;
		}
	}

	// TODO: Adjust elevation angle
	if (key == 'w')
	{
		elevation += del;
		if (elevation > 180.0)
		{
			elevation = 180.0;
		}
	}
	else if (key == 's')
	{
		elevation -= del;
		if (elevation < 0.0)
		{
			elevation = 0.0;
		}
	}

	// TODO: Adjust radius (zoom)
	if (key == 'x')
	{
		radius += dr;
	}
	else if (key == 'z')
	{
		radius -= dr;
		if (radius < min_radius)
		{
			radius = min_radius;
		}
	}

	// TODO: Compute cartesian camera position
	eye[X] = (GLfloat)(radius*sin(azimuth*DEG2RAD)*sin(elevation*DEG2RAD));
	eye[Y] = (GLfloat)(radius*cos(elevation*DEG2RAD));
	eye[Z] = (GLfloat)(radius*cos(azimuth*DEG2RAD)*sin(elevation*DEG2RAD));

	// <esc> quits
	if (key == 27)
	{
		exit(0);
	}

	// Redraw screen
	glutPostRedisplay();
}

// Routine to create display lists
void create_lists()
{
	// TODO: Create colorcube display list
	glNewList(CUBE, GL_COMPILE);
		glPushAttrib(GL_CURRENT_BIT);
		colorcube();
		glPopAttrib();
	glEndList();

}

// Reshape callback
void reshape(int w, int h)
{
	// Set new screen extents
	glViewport(0,0,w,h);
}

// Routine to draw cube
void colorcube()
{
	// Top face
	quad(cube[4],cube[7],cube[6],cube[5],colors[0],colors[0],colors[0],colors[0]);

	// Bottom face
	quad(cube[0],cube[1],cube[2],cube[3],colors[1],colors[1],colors[1],colors[1]);

	// Left face
	quad(cube[0],cube[3],cube[7],cube[4],colors[2],colors[2],colors[2],colors[2]);

	// Right face
	quad(cube[1],cube[5],cube[6],cube[2],colors[3],colors[3],colors[3],colors[3]);

	// Front face
	quad(cube[2],cube[6],cube[7],cube[3],colors[4],colors[4],colors[4],colors[4]);

	// Back face
	quad(cube[0],cube[4],cube[5],cube[1],colors[5],colors[5],colors[5],colors[5]);
}

// Routine to draw (outlined) quadrilateral face
void quad(GLfloat v1[], GLfloat v2[], GLfloat v3[], GLfloat v4[],
	      GLfloat c1[], GLfloat c2[], GLfloat c3[], GLfloat c4[])
{
	// Draw face
	glBegin(GL_POLYGON);
		glColor3fv(c1);
		glVertex3fv(v1);
		glColor3fv(c2);
		glVertex3fv(v2);
		glColor3fv(c3);
		glVertex3fv(v3);
		glColor3fv(c4);
		glVertex3fv(v4);
	glEnd();

	// Draw outline
	glColor3f(0.0f,0.0f,0.0f);
	glBegin(GL_LINE_LOOP);
		glVertex3fv(v1);
		glVertex3fv(v2);
		glVertex3fv(v3);
		glVertex3fv(v4);
	glEnd();
}
